package alex.carmessenger;

import android.app.Activity;
import android.content.Intent;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
/*
*the client activity is how the user connects to the server device so that they can communicate with other drivers
*a new thread is started from the activity which does the processing the background task rather than on the main ui thread.
*by inputting the IP address of the server device, the user connects to the listening server device. 
*once the connection is set up, the message is transmitted between the devices so that the server user receives the message.
*/
public class Client extends Activity {
    private EditText serverIp;
    private Button connectPhones;
    private String serverIpAddress = "";
    private boolean connected = false;
    private String message= "";
    private String regNum = "";

    private Handler handler = new Handler(Looper.getMainLooper());
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_client);

        message = getIntent().getExtras().getString("message");
        regNum = getIntent().getExtras().getString("regNumber");
        serverIp = (EditText) findViewById(R.id.server_ip);
        connectPhones = (Button) findViewById(R.id.connect_phones);
        connectPhones.setOnClickListener(connectListener);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_actions, menu);

        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_homeButton:
                Intent mainMenuIntent = new Intent(this, MainMenu.class);
                startActivity(mainMenuIntent);
                finish();
                return true;

            case R.id.action_logout:
                Intent logoutIntent = new Intent(this, HomeScreen.class);
                startActivity(logoutIntent);
                finish();
                return true;

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);

        }
    }

    public void menuIntentClient(View view){
        Intent mainMenuIntent = new Intent(this, MainMenu.class);
        startActivity(mainMenuIntent);
        finish();
    }

    private View.OnClickListener connectListener = new View.OnClickListener() {

        @Override
        public void onClick(View v) {
            if (!connected) {
                serverIpAddress = serverIp.getText().toString();
                if (!serverIpAddress.equals("")) {
                    Thread cThread = new Thread(new ClientThread());
                    cThread.start(); //start a new thread to handle the client requests
                }
            }
        }
    };

    public class ClientThread implements Runnable {
        public Handler mHandler;

        public void run() {
            Looper.prepare();
            mHandler = new Handler();

            try {
                InetAddress serverAddr = InetAddress.getByName(serverIpAddress);
                Log.d("ClientActivity", "C: Connecting...");
                Socket socket = new Socket(serverAddr, ServerActivity.SERVERPORT);
                connected = true;
                while (connected) {
                    try {
                        Log.d("ClientActivity", "C: Sending command.");
                        PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket
                                .getOutputStream())), true);
                        // THE COMMANDS being issued
                        out.println(message + ":" + regNum);
                        Log.d("ClientActivity", "C: Sent.");
                        Log.d(message, "Message Recieved");
                        out.flush();
                        break;
                    } catch (Exception e) {
                        Log.e("ClientActivity", "S: Error", e);
                        e.getStackTrace();
                    }
                }
                socket.close();
                Log.d("ClientActivity", "C: Closed.");
                Toast.makeText(Client.this, "Connection Closed", Toast.LENGTH_SHORT).show();
            } catch (Exception e) {
                Log.e("ClientActivity", "C: Error", e);
                connected = false;
            }
        }
    }
}
