package alex.carmessenger;

import android.*;
import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.FragmentActivity;
import android.os.Bundle;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.drive.Permission;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.UiSettings;
import com.google.android.gms.maps.model.CircleOptions;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import java.util.Date;


public class ReportAccident extends AppCompatActivity implements OnMapReadyCallback {
    MapView mapView;
    private GoogleMap mMap;
    private static final int PERMISSION_REQUEST_CODE = 101;
    DBAdapter db = new DBAdapter(this);
    EditText streetName;
    LocationRequest mLocationRequest;
    GoogleApiClient mGoogleApiClient;
    Double mLatitude;
    Double mLongitude;
    String street;
    int count = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_report_accident);
        Thread.setDefaultUncaughtExceptionHandler(new ErrorHandler(this));
        db.open();

        SupportMapFragment mapFragment =
                (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

        if (mGoogleApiClient == null) {
            mGoogleApiClient = new GoogleApiClient.Builder(this)
                    .addApi(LocationServices.API)
                    .build();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_actions, menu);

        return super.onCreateOptionsMenu(menu);
    }
    private boolean checkPermission(){
        int result = ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION);
        if(result == PackageManager.PERMISSION_GRANTED){
            return true;
        }
        else{
            return false;
        }
    }
    private void requestPermission(){
        if(ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.ACCESS_FINE_LOCATION)){
            Toast.makeText(this,"GPS PERMISSION IS NEEDED TO LOCATE ANY ACCIDENTS", Toast.LENGTH_SHORT);
        }
        else{
            ActivityCompat.requestPermissions(this, new String[] {Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_REQUEST_CODE);
        }

    }
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults){
        switch(requestCode){
            case PERMISSION_REQUEST_CODE:
                if(grantResults.length >0 && grantResults[0] == PackageManager.PERMISSION_GRANTED){
                    Toast.makeText(this, "You can now access location data", Toast.LENGTH_SHORT);
                }
                else{
                    Toast.makeText(ReportAccident.this, "You can not access location data", Toast.LENGTH_SHORT).show();
                }
        }
    }

    /**
     * Manipulates the map once available.
     * This callback is triggered when the map is ready to be used.
     * This is where we can add markers or lines, add listeners or move the camera.
     * If Google Play services is not installed on the device, the user will be prompted to install
     * it inside the SupportMapFragment. This method will only be triggered once the user has
     * installed Google Play services and returned to the app.
     */
    @Override
    public void onMapReady(final GoogleMap googleMap) {
        checkPermission();
        requestPermission();
        mMap = googleMap;
//        mMap.getUiSettings();
//        mMap.setTrafficEnabled(true);
//        mMap.setMapType(1);

        streetName = (EditText)findViewById(R.id.enterStreetNameActivity);
        if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_FINE_LOCATION) ==
                PackageManager.PERMISSION_GRANTED &&
                ContextCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_COARSE_LOCATION) ==
                        PackageManager.PERMISSION_GRANTED) {
            mMap.setMyLocationEnabled(true);
            mMap.getUiSettings().setMyLocationButtonEnabled(true);
            mMap.getUiSettings().setZoomControlsEnabled(true);
            mMap.setMapType(1);
        }
        else {
            ActivityCompat.requestPermissions(this, new String[] {Manifest.permission.ACCESS_FINE_LOCATION}, PERMISSION_REQUEST_CODE);
        }
        Location currentLocation = LocationServices.FusedLocationApi.getLastLocation(mGoogleApiClient);
        if (currentLocation != null) {
            mLatitude = currentLocation.getLatitude();
            mLongitude = currentLocation.getLongitude();

            float zoomLevel = (float) 16.00;
            LatLng latLng = new LatLng(mLatitude, mLongitude);
            mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, zoomLevel));
            mMap.addMarker(new MarkerOptions().position(new LatLng(mLatitude, mLongitude)).title("You are here!").snippet
                    ("Consider yourself located"));
        }
        else{
            Double longitude = -3.186035;
            Double latitude = 51.477748;
            streetName.setText("");
            float zoomLevel = (float) 16.00;
            LatLng latLng = new LatLng(latitude, longitude);
            mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, zoomLevel));
            mMap.addMarker(new MarkerOptions().position(new LatLng(latitude, longitude)).title("You are here!").snippet
                    ("Consider yourself located"));
        }



        googleMap.setOnMapClickListener(new GoogleMap.OnMapClickListener(){
            @Override
            public void onMapClick(LatLng arg0){
                street = streetName.getText().toString();
                if(count <=3 && !street.isEmpty() ) {
                        Marker marker = mMap.addMarker(new MarkerOptions().position(arg0).title("New Accident"));
                        LatLng lat = marker.getPosition();
                        Double latPosition = lat.latitude;
                        Double lngPosition = lat.longitude;
                        String latitude = latPosition.toString();
                        String longitude = lngPosition.toString();
                        Date newDate = new Date(System.currentTimeMillis());
                        String myDate = newDate.toString();
                        db.insertMarker(latitude, longitude, street, myDate.toString());
                        count += 1;
                }

                else{
                    Toast.makeText(ReportAccident.this, "You have created too many markers. Or enter a street name.",
                            Toast.LENGTH_SHORT).show();
                }
//                android.util.Log.i(marker.getPosition().toString(), streetName.getText().toString());
//                android.util.Log.i("time is : " , myDate);

            }
        });

    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_homeButton:
                Intent mainMenuIntent = new Intent(this, MainMenu.class);
                startActivity(mainMenuIntent);
                finish();
                return true;

            case R.id.action_new_message:
                Intent sendMessageIntent = new Intent(this, sendMessage.class);
                startActivity(sendMessageIntent);
                finish();
                return true;

            case R.id.action_logout:
                Intent logoutIntent = new Intent(this, HomeScreen.class);
                startActivity(logoutIntent);
                finish();
                return true;

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);

        }
    }


}
