package alex.carmessenger;
/*
*The Server activity class, implements socket programming that creates a new thread where a
* user listens for commands on the activity.
* New commands are sent from a client device that has connected to the server device via inputting
* this server device's IP address.
 */
import android.app.Activity;
import android.content.Intent;
import android.os.Handler;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.StringTokenizer;

public class ServerActivity extends Activity {

    private TextView serverStatus;
    private TextView serverUpdates;
    private String message= "";
    private String regNum = "";
    // DEFAULT IP
    public static String SERVERIP = "10.0.2.15";
    // DESIGNATE A PORT
    public static final int SERVERPORT = 8080;
    private Handler handler = new Handler();
    private ServerSocket serverSocket;
    DBAdapter db;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_server);
        db = new DBAdapter(ServerActivity.this);
        db.open();
        serverStatus = (TextView) findViewById(R.id.server_status);
        serverUpdates = (TextView)findViewById(R.id.serverTextMessage);
        SERVERIP = getLocalIpAddress();

        Thread fst = new Thread(new ServerThread());
        fst.start();
    }
//creates a new thread that does the handling in a seperate thread to the main UI thread
public class ServerThread implements Runnable {

    public void run() {
        try {
            if (SERVERIP != null) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        serverStatus.setText("Listening on IP: " + SERVERIP);
                    }
                });
                serverSocket = new ServerSocket(SERVERPORT);
                while (true) {
                    // LISTEN FOR INCOMING CLIENTS
                    Socket client = serverSocket.accept();
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            serverStatus.setText("Connected.");
                            serverUpdates.setText("Incoming Messages...");
                        }
                    });
                    //Gets the input message from the client device
                    try {
                        BufferedReader in = new BufferedReader(new InputStreamReader(client.getInputStream()));
                        String line = null;
                        while ((line = in.readLine()) != null) {
                            Log.d("ServerActivity", line);
                            serverUpdates.setText("Message Received - " + line.toString());

                            handler.post(new Runnable() {
                                @Override
                                public void run() {

                                    threadMsg(serverUpdates.getText().toString());
                                    //get messages from client
                                    Toast.makeText(ServerActivity.this, "New Message", Toast.LENGTH_SHORT).show();
                                }
                            });
                        }
                        break;
                    } catch (Exception e) {
                        handler.post(new Runnable() {
                            @Override
                            public void run() {
                                serverStatus.setText("Oops. Connection interrupted. Please reconnect your phones.");
                            }
                        });
                        e.printStackTrace();
                    }

                }
            } else {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        serverStatus.setText("Couldn't detect internet connection.");
                    }
                });
            }
        } catch (Exception e) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    serverStatus.setText("Error");
                }
            });
            e.printStackTrace();
        }

    }

}
    private void threadMsg(String msg){
        Message msgObj = handler.obtainMessage();
        Bundle b = new Bundle();
        b.putString("message", msg);
        msgObj.setData(b);
        handlerer.sendMessage(msgObj);
    }
//handler that handles the message received from the client activity.
// the message is then seperated into the different fields and added to the server's database. 
    private final Handler handlerer = new Handler(){
        public void handleMessage(Message msg){
            String aResponse = msg.getData().getString("message");
            StringTokenizer tokens = new StringTokenizer(aResponse, "-"); //seperates the words in the tokenizer after the dash. 
            String first = tokens.nextToken();
            String message= tokens.nextToken();
            StringTokenizer secondTokens = new StringTokenizer(aResponse, ":");
            String firstSecond = secondTokens.nextToken();
            String regNum = secondTokens.nextToken();
            Calendar c = Calendar.getInstance();
            SimpleDateFormat df = new SimpleDateFormat("yyyy-mm-dd HH:mm:ss");
            String currentDateTimeString = df.format(c.getTime());

            if ((null != aResponse)) {
                Log.d("reg num = ", regNum);
                db.insertMessage(message, currentDateTimeString, regNum );
                Toast.makeText(getBaseContext(), "Message Added to db", Toast.LENGTH_SHORT).show();
            }
            else{
            Toast.makeText(getBaseContext(),"Not Got Response From Server.",Toast.LENGTH_SHORT).show();
            }

        }
    };


    // GETS THE IP ADDRESS OF YOUR PHONE'S NETWORK
    private String getLocalIpAddress() {
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
                NetworkInterface intf = en.nextElement();
                for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
                    InetAddress inetAddress = enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress()) { return inetAddress.getHostAddress().toString(); }
                }
            }
        } catch (SocketException ex) {
            Log.e("ServerActivity", ex.toString());
        }
        return null;
    }

    public void menuIntent(View view){
        Intent mainMenuIntent = new Intent(this, MainMenu.class);
        startActivity(mainMenuIntent);
        finish();
    }

    @Override
    protected void onStop() {
        super.onStop();
        try {
            //CLOSE THE SOCKET UPON EXITING
            serverSocket.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

};


